/*
 * MAX : adc520.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for the fast 4 channel 12bit ADC Hytec 520M
 *
 * This file is part of Max data acquisition software
 * (c) 2001 by S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

extern "C" {
#include "camacs.h"
}

#undef DEBUG_DEV_ADC520
//#define DEBUG_DEV_ADC520

#define ADC520_ID_CODE 55
#define ADC520_CHANMAX 1

// global variables
static int  N, qx, TLAM, shots;

/******************* function(s) common to all channels **********************/
int adc520_readout ( int channel, Scheduler *parameters )
{
  int  d;
  unsigned int dummy;

  	qx = camac(0,channel,N,&dummy);   //readout adc
  	d = (int)dummy;

  //parameters->store((parameters->act_channel() % shots == 0)?((d <=2048?(2048-d):(4096-(d-2048)))):(-1));  //save measured or no-meas value
	parameters->store((2048-d));

#ifdef DEBUG_DEV_ADC520
  printf("\nadc520_%02d: %s data stored (%d)",channel,(parameters->act_channel() % shots == 0)?("real"):("virtual"),d);
#endif

  return 1;
}
/*****************************************************************************/

//init function
//called each time before meausurement starts
int adc520_init(List *device_data, Scheduler *parameters)
{	
  unsigned int x;
  char errmsg[81];

  if (camac_init() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't initialize device "))
      return 0;

  printf("\ninitializing adc520");
  device_data->getvar(&N,"adc520","N");
  device_data->getvar(&TLAM,"all","timeout");
  device_data->getvar(&shots,"adc520","shots");

  //check if adc520 really present in CAMAC slot N
  qx = camac(1,0,N,&x);
	
  if (qx < 3) {
    sprintf(errmsg,"adc520_init(): qx=%d returned ",qx);
    if (!parameters->error(E_CAMAC,errmsg))
      return 0;   //return FALSE -> measurement stops
  }

  if (x != ADC520_ID_CODE) {
	printf("Control Register = %d\nSetting to ",x);
	unsigned int tmp = ADC520_ID_CODE;
	camac(17,0,N,&tmp);
  	camac(1,0,N,&x);
	printf("%d \n",x);
	if (x != ADC520_ID_CODE) {
    		sprintf(errmsg,"adc520_init(): Wrong device ID (%d <> %d) ",x,ADC520_ID_CODE);
    		if (!parameters->error(E_CAMAC,errmsg))
      			return 0;
	}
  }

  return 1;
}

int adc520_pre ( List *device_data, Scheduler *parameters )
{
  // nothing to do...

  return 1;
}

int adc520 ( List *device_data, Scheduler *parameters )
{
  int i;
  char label[6];

  for (i = 0; i < ADC520_CHANMAX; i++) {
    sprintf(label,"ch%02d",i);
    if (device_data->enabled("adc520",label)) {
      if ( ! adc520_readout(i,parameters))
	return 0;
    }
#ifdef DEBUG_DEV_ADC520
    else {
      printf("\nadc520_%02d: Not enabled - skipping readout ",i);
    }
#endif
  }

  return 1;
}

int adc520_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing adc520 ");
  if (camac_close() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't close device"))
      return 0;

  return 1; //function has to return a value -> 1 otherwise measurement stops
}
