/*
 * MAX : srs_func.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for the Stanford Function Generator Model DS345
 *
 * M. Dammasch 14.07.1998 implementation of gpib.cpp-interface functions
 * M. Dammasch 05.11.1998 upgrade to v2.0beta
 * M. Dammasch 15.12.1998 error()-handling changed (only 0 returns)
 * 
 * This file is part of Max data acquisition software
 * (c) 1998 by C. Otto & M. Dammasch
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//this header has always to be included !!
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
//#include <ib.h>
#include <gpiberr.h>
#include <gpib.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"


#define MAXLEVEL 10            //max. Referencelevel
#define MAXFREQ 30000000       //max. frequency 30MHz

static GPIB_Device  srs;       //Gpib-device structure
static double       level;     //Referencelevel
static double       freq;      //Frequency
static double       amp_step;  //step for Amplitude
static double       freq_step; //step for Frequency

int srs_function_init(List *device_data, Scheduler *parameters)
{
  char id_str[GPIB_STRMAX+1];

  //init Stanford Function Generator
  srs = gpib_init("srs_ds345",IB_CLEAR|IB_REM|IB_IFC);
  if (srs.stat != GPIB_OK)
  {
    if ( !parameters->error(E_GPIB,"GPIB init err"))
      return 0;
  }

  if ( !gpib_write(&srs,"*IDN?"))
  {
    if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
      return 0;
  }
  if ( !gpib_read(&srs,id_str,GPIB_STRMAX))
  {
    if ( !parameters->error(E_GPIB,"read error on 'SRS_DS345' "))
      return 0;
  }
  printf("\nDevice-ID: %s",id_str);

  device_data->getvar(&level,"srs_function","level");  //Referencelevel
  device_data->getvar(&freq,"srs_function","freq");       //Startfreq
  if (level > MAXLEVEL)
    level = MAXLEVEL;
  if (freq > MAXFREQ)
    freq = MAXFREQ;

  printf("\nLevel:%G\tFreq:%G",level,freq);

  printf("\ninitializing srs_function...");

/* if ( !gpib_write(&srs,"%s%lf%s","AMPL",level,"VR"))  //set output amplitude to level Vrms
  {
    if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
    return 0;
  }
*/

  if ( !gpib_write(&srs,"%s%lf%s","AMPL",level,"VP"))  //set output amplitude to level Vpp
  {
    if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
      return 0;
  }

  if ( !gpib_write(&srs,"%s%lf","FREQ",freq))              //set frequency to freq Hz with HZ resolution
  {
    if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
      return 0;
  }

  printf(" done ");

  return 1;
}


int srs_function_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing srs_function... done ");
  return 1; //function has to return a vlue -> >1 otherwise measurement stops
}





//device function for Stanford Function Generator
int srs_function_pre(List *device_data, Scheduler *parameters)
{
  //increase frequency

  device_data->getvar(&freq_step,"srs_function","freq_step");
  if (freq_step!=0)
    {
      freq+=freq_step;
      if (freq> MAXFREQ)
	freq=MAXFREQ;

      if ( !gpib_write(&srs,"%s%lf", "FREQ",freq))       //write freq
      {
	if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
	  return 0;
      }
      return 1;
    }

  //increase amplitude

  device_data->getvar(&amp_step,"srs_function","amp_step");
  if (amp_step!=0)
    {
      level+=amp_step;
      if (level> MAXLEVEL)
	level=MAXLEVEL;

/*      if ( !gpib_write(&srs,"%s%lf%s","AMPL",level,"VR"))        //set output amplitude to level Vrms
      {
        if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
	  return 0;
      }
*/
      if ( !gpib_write(&srs,"%s%lf%s","AMPL",level,"VP"))        //set output amplitude to level Vpp
      {
	if ( !parameters->error(E_GPIB,"write error on 'SRS_DS345' "))
	  return 0;
      }
      return 1;
    }

  printf("\nSRS: Warning: Bad parameters (AmpStep: %G, FreqStep: %G) ",amp_step,freq_step);
  return 1;
}

//device function for Stanford Function Generator
int srs_function(List *device_data, Scheduler *parameters)
{

  return 1;
}

/* of  Stanford Function Generator */
