/*
 * MAX : camacs.c, Son Sep 29 10:18:09 CEST 2002
 * 
 * CAMAC interface for use with the MAX data aquisition program
 *
 *    11.12.1988  M.Voelkel
 *    31.01.1994  D.Rohloff    geaendert fuer microsys ieee
 *    04.01.1996  J.Ludwig     Programmierstil geaendert
 *    10.07.1997  C.Otto       fr Linux umgeschrieben
 *    08.04.1998  M. Dammasch  auf mehrere IEEE-Devices verallgemeinert
 *    16.08.2002  S. Gerlach	cleared programming style
 *    02.09.2002  S. Gerlach	unsigned char daten[4] !!!!!!!!!	
 *    29.09.2002  S. Gerlach	_CAMAC_IEEE488_
 *
 * - CAMAC Functions for the LeCroy 8901 GPIB Crate controller: 
 * 		_CAMAC_GPIB_		(Linux-GPIB Driver)
 		_CAMAC_IEEE488_		(INES-GPIB Driver)
 * - CAMAC Functions for the Wiener PC16 CAMAC Crate controller: 
 		_CAMAC_C111A_		(c111a Driver)
 *   This driver is based on the c111a driver for the Wiener PC16Turbo card
 *   (c) Giovanni Organtini, organtini@roma1.infn.it
 */

#include <stdio.h>
#include <stdlib.h>
#include "camacs.h"

#ifdef _CAMAC_GPIB_
#include <ib.h>
#include "gpiberr.h"
#endif
#ifdef _CAMAC_C111A_
#include <c111a.h>
#include <naf.h>
#endif
#ifdef _CAMAC_IEEE488_
#include <gpibfx.h>
#endif

//#define DEBUG_CAMAC

#ifdef DEBUG_CAMAC
        #define INFO(text) printf(text)
#else
        #define INFO(text)
#endif

// global variables
#ifdef _CAMAC_GPIB_
int  Eid_Cam;  // GPIB address of LeCroy-CAMAC-Controller
#endif
#ifdef _CAMAC_C111A_
int  cc16_fd;  // file descriptor of PC16 C111A-CAMAC-Controller
#endif
#ifdef _CAMAC_IEEE488_
int pad = 2;		// GPIB address of LeCroy-CAMAC-Controller
int TMO = 10000;	// maximum : 2147000
#endif

int  CamInit;  // CAMAC initialization flag

//////////////////////////////////////////////////////////////////////////
int  camac_init(void) {
#ifdef _CAMAC_GPIB_
	char d[5];
#endif
#ifdef _CAMAC_C111A_
	int q, x, data;
#endif
#ifdef _CAMAC_IEEE488_
	int count=4;
	byte ans[4];
#endif

	if (CamInit) // exit if already initialized
 		return 1;
	CamInit = 0; // set CAMAC not initialized

#ifdef _CAMAC_GPIB_
  	printf("\n***** Connecting GPIB Device 'LeCroy' *****\n");

 	if((Eid_Cam = ibfind("LeCroy")) & ERR){
    		gpiberr("ibfind error");
    		return 0;
  	}

#ifdef DEBUG_CAMAC
  	printf("LeCroy : GPIB-Address is set to #%d\n",Eid_Cam);    
  	printf("LeCroy : Send IFC\n");  // ibsic from Lib.
#endif
		
  	if( ibsic(Eid_Cam) & ERR){
    		gpiberr("ibsic err");
    		return 0;
  	}

 	INFO("LeCroy : Send REM\n");  // ibsre from Lib.
  	if( ibsre(Eid_Cam,1) & ERR){
    		gpiberr("ibsre err");
    		return 0;
  	}

  	INFO("LeCroy : Clearing GPIB bus\n");  // ibclr from Lib.
  	if( ibclr(Eid_Cam) & ERR){
    		gpiberr("ibclear error");
    		return 0;
  	}
#endif	// CAMAC_GPIB
  
#ifdef _CAMAC_C111A_
  	printf("\n\n***** Connecting CAMAC-Controller 'C111A' ***** ");
  
  	if ((cc16_fd = c111aopen(C111A_ADDR)) <= 0) {  // failed
    		switch (cc16_fd) {
    		case -2 :
      			fprintf(stderr,"\nCamac_Init(): Couldn't get file handle for C111A@%d ",C111A_ADDR); 
      			break;
    		case -1 :
      			fprintf(stderr,"\nCamac_Init(): Invalid crate address (%d) for C111A ",C111A_ADDR); 
      			break;
    		default :
      			fprintf(stderr,"\nCamac_Init(): Illegal file descriptor (%d) for C111A@%d ",cc16_fd,C111A_ADDR); 
      			break;
    		}
    		return 0;
  	}
  	printf("\n'C111A' file handle is set to #%d ",cc16_fd);
#endif
  
	/****** Init Crate Controller *******/
#ifdef _CAMAC_GPIB_
  	INFO("LeCroy : Send CAMAC-Clear(C) and CAMAC-Initialize(Z)\n");
  	if( ibwrt (Eid_Cam,"\x23",1)& ERR ){  // Send both CLEAR and INITIALIZE
    		gpiberr("write error");
    		return 0;
  	}
  	if( ibrd (Eid_Cam,d,4)& ERR){
    		gpiberr("read error");
    		return 0;	
  	}
  
  	INFO("LeCroy : Set data mode: ");
  	if( ibwrt (Eid_Cam,"\x64",1)& ERR ){  // set data transfer mode (24bit)
    		gpiberr("write error");
    	return 0;
  	}
  	if( ibrd (Eid_Cam,d,4)  & ERR){
    		gpiberr("read error");
    		return 0;
  	}
  	INFO("24-bit read normal transfer\n");
#endif //_CAMAC_GPIB_

#ifdef _CAMAC_C111A_
  	q = x = data = 0;
  	printf("\nC111A_%d.25: F(36)A(1)[%d]: Clear I, C cycle ",cc16_fd,data);
 	c111a(cc16_fd, 25, 1, 36, &q, &x, &data);
  	if (x == 1)  // okay
    		printf("done\n");
  	else {
    		printf("failed (QX: %d <%d%d>\t D: %d (0x%x)) ",2*q+x,q,x,data,data); 
    		fprintf(stderr,"\nCamac_Init(): C111A_%d.25: Couldn't execute F(36)A(1) ",cc16_fd);
    		return 0;
  	}

  	q = x = data = 0;
  	printf("\nC111A_%d.25: F(38)A(1)[%d]: Clear I, Z, C cycle ",cc16_fd,data);
  	c111a(cc16_fd, 25, 1, 38, &q, &x, &data);
  	if (x == 1)  // okay
    		printf("done\n");
  	else {
    		printf("failed (QX: %d <%d%d>\t D: %d (0x%x)) ",2*q+x,q,x,data,data); 
    		fprintf(stderr,"\nCamac_Init(): C111A_%d.25: Couldn't execute F(38)A(1) ",cc16_fd);
    		return 0;
  	}
#endif

#ifdef _CAMAC_IEEE488_
  	printf("\n***** Connecting GPIB Device 'LeCroy' with IEEE488  *****\n");
	ieinit();
	ieoutput(pad,"*RST\n",5,TMO);   // reset
	ieoutput(pad,"*CLS\n",5,TMO);   // clear status reg
	ieoutput(pad,"\x23",3,TMO);   	// initialize LeCroy
	ieenter(pad,ans,&count,TMO);
	ieoutput(pad,"\x64",3,TMO);	// init crate controller
	ieenter(pad,ans,&count,TMO);
#endif

  	CamInit = 1;
  	printf("\n***** Completed Successfully *****\n\n");

  	return 1;
 }

////////////////////////////////////////////////////////////////////////////
int camac_close () {
#ifdef _CAMAC_GPIB_
  	char d[5];
#endif
#ifdef _CAMAC_C111A_
  	int q, x, data;
#endif
  
  	if (CamInit) { 
#ifdef _CAMAC_GPIB_
    		INFO("LeCroy : Send CAMAC-Clear(C) ...\n");
    		if( ibwrt (Eid_Cam,"\x22",1)& ERR ){  // Send CLEAR 
      			gpiberr("write error");
      			return 0;
    		}

    		if( ibrd (Eid_Cam,d,4)& ERR){
 			gpiberr("read error");
     			return 0;	
    		}
    		INFO("LeCroy : Clearing GPIB bus\n");  // ibclr aus Lib.
    		if( ibclr(Eid_Cam) & ERR){
      			gpiberr("ibclear error");
      			return 0;
    		}
#endif
    
#ifdef _CAMAC_C111A_
    		q = x = data = 0;
    		printf("\nC111A_%d.25: F(38)A(1)[0]: Clear I, Z, C cycle ",cc16_fd);
    		c111a(cc16_fd, 25, 1, 38, &q, &x, &data);
    		if (x == 1)  // okay
      			printf("done\n");
    		else {
      			printf("failed (QX: %d <%d%d>\t D: %d (0x%x)) ",2*q+x,q,x,data,data); 
      			fprintf(stderr,"\nCamac_Close(): C111A_%d.25: Couldn't execute F(38)A(1) ",cc16_fd);
      			return 0;
    		}
    
    		c111aclose(cc16_fd);
#endif
  	}
  
#ifdef _CAMAC_IEEE488
	INFO("IEEE488 : iedeinit()\n");
	ieoutput(pad,"*CLS\n",5,TMO);   	// clear status reg
	iedeinit();
#endif
  
  	CamInit = 0;
  	return 1;
}

/////////////////////////////////////////////////////////////////////////////////////////
int camac(int f,int a,int n,unsigned int *d) {
#ifdef _CAMAC_GPIB_
	unsigned char camac_befehl[6];	// commmand string for CAMAC call
  	unsigned char camac_daten[4];	// datastring for CAMAC answer
  	int anzahl_bytes;		// number of bytes for transmission

  	if (f > CAMAC_BEFEHL) {		// CAMAC command invalid
      		printf("Wrong CAMAC settings\n");
      		exit(0);
    	}
  	else {
      		camac_befehl[CAMAC_F] = (char) (f & 0xff);     // lower byte of f in command string
      		camac_befehl[CAMAC_N] = (char) (n & 0xff);     // lower byte of n in command string
      		camac_befehl[CAMAC_A] = (char) (a & 0xff);     // lower byte of a in command string
      
	      	switch (f & CAMAC_BEFEHLS_MASKE) {
		case CAMAC_READ:
		case CAMAC_SET_STATUS_A:
		case CAMAC_SET_STATUS_B:
		  	anzahl_bytes = 3;
		  	break;
		case CAMAC_WRITE:
		  	camac_befehl[CAMAC_D1] = (char) (*d & 0xff);
		 	camac_befehl[CAMAC_D2] = (char) ((*d & 0xff00 ) >> 8);
		  	camac_befehl[CAMAC_D3] = (char) ((*d & 0xff0000 ) >> 16);
		  	anzahl_bytes = 6;
		  	break;
		default:
		  	anzahl_bytes = 0;
		  	printf("Wrong CAMAC settings\n");
		  	exit(0);
		  	break;
		}
      		if( ibwrt(Eid_Cam, camac_befehl, anzahl_bytes) & ERR ){
			printf("Warning, error occured!\n");
			gpiberr("write error");
			exit(1);
      		}
     	 
      		if( ibrd (Eid_Cam, camac_daten, 4) & ERR){
			printf("Warning, error occured!\n");
			gpiberr("read error");
			exit(1);
      		}

#ifdef DEBUG_CAMAC
		printf("\nCAMAC@%02d.%02d: F(%02d)A(%02d)[%d] -> QX: %d   D: %d ",Eid_Cam,n,f,a,*d,
			(int)camac_daten[3],(int) (camac_daten[0] | ( camac_daten[1] << 8 ) | ( camac_daten[2] << 16 )));
#endif
             
		if ((f & CAMAC_BEFEHLS_MASKE) == CAMAC_READ) {	// data additional to QX
 			*d=camac_daten[0] | ( camac_daten[1] << 8 ) | ( camac_daten[2] << 16 );
		}
      		return ( (int) camac_daten[3]);   // return value : QX (QX = 3 success)
	}
#endif // _CAMAC_GPIB_

#ifdef _CAMAC_C111A_
  	int q=0, x=0;
  
#ifdef DEBUG_CAMAC
  	printf("\nC111A_%d.%02d: F(%d)A(%d)[%d] ",cc16_fd,n,f,a,*d);
#endif      
  	c111a(cc16_fd, n, a, f, &q, &x,(int *) d);
#ifdef DEBUG_CAMAC
  	printf("-> QX: %d <%d%d>\t D: %d (0x%x) ",2*q+x,q,x,*d,*d);
#endif               
  	return(2*q+x);
#endif // _CAMAC_C111A_

#ifdef _CAMAC_IEEE488_
	uint cnt;
	unsigned char camac_befehl[6];	// command string for CAMAC WRITE call
	unsigned char camac_read[4];	// command string for CAMAC READ call
	unsigned char camac_daten[6];	// data string for CAMAC answer

	if (f > CAMAC_BEFEHL) {		// camac command invalid
		exit(printf("Wrong CAMAC instruction f=%d due to\n"));
	}
	else {
		camac_befehl[CAMAC_F] = (char) (f & 0xff);	//lower byte of f in command string
		camac_befehl[CAMAC_N] = (char) (n & 0xff);	//lower byte of n in command string
		camac_befehl[CAMAC_A] = (char) (a & 0xff);	//lower byte of a in command string

		switch (f & CAMAC_BEFEHLS_MASKE) {
		case CAMAC_READ:
		case CAMAC_SET_STATUS_A:
		case CAMAC_SET_STATUS_B:
			camac_read[0]=camac_befehl[0];
			camac_read[1]=camac_befehl[1];
			camac_read[2]=camac_befehl[2];
			camac_read[3]=0;
#ifdef DEBUG_CAMAC
			printf("camac.c : ieoutput(%d,[%d,%d,%d,%d],%d)\n",
			 	pad,camac_read[0],camac_read[1],camac_read[2],
				camac_read[3],TMO);
#endif
			if( ieoutput(pad, camac_read,4,TMO) != 0){
				printf("ieoutput error in CAMAC_WRITE(4)\n");
				exit(1);
			}
			break;
		case CAMAC_WRITE:
			camac_befehl[CAMAC_D1] = (char) (*d & 0xff);
			camac_befehl[CAMAC_D2] = (char) ((*d & 0xff00 ) >> 8);
			camac_befehl[CAMAC_D3] = (char) ((*d & 0xff0000 ) >> 16);
			camac_befehl[6] = 0;
#ifdef DEBUG_CAMAC
			 printf("camac.c : ieoutput(%d,[%d,%d,%d,%d,%d,%d],%d)\n",
				pad,camac_befehl[0],camac_befehl[1],camac_befehl[2],
				camac_befehl[3],camac_befehl[4],camac_befehl[5],TMO);
#endif
			 if(ieoutput(pad, camac_befehl,6,TMO) != 0){
				printf("ieoutput error in CAMAC_WRITE(6)\n");
				exit(1);
			}
			break;
		default:
			exit(printf("Wrong CAMAC setting f=%d due to\n"));
			break;
		}

		// reading camac answer
		cnt=4;
		camac_daten[0] = 0;
		INFO("camac.c : ieenter()\n");
		if( ieenter(pad, camac_daten, &cnt, TMO) != 0){
			printf(" ieenter error\n");
		}

#ifdef DEBUG_CAMAC
		printf("CAMAC N,F,A,D = %d,%d,%d,%d  Response %d  %d\n",n,f,a,*d,(int) camac_daten[3],
			(int) (camac_daten[0] | ( camac_daten[1] << 8 ) | ( camac_daten[2] << 16 )) );
#endif
		
		if ((f & CAMAC_BEFEHLS_MASKE) == CAMAC_READ) { // reading data additional to qx
			*d  =   camac_daten[0] |
				( camac_daten[1] << 8 ) |
				( camac_daten[2] << 16 ) ;
		}

		return ( (int) camac_daten[3]);               // return QX. OK if qx=3
	}
#endif
}
