/*
	ines ieee488 informations for low level access
	Version 4.2
	Copyright (C) ines GmbH Koeln 1987 - 2001
	10.10.95 added:
		generic event handling for select()/poll()

	
 	28.9.2001:
	 	initial linux version 
		- no device support	
		- no select/poll support

*/

#ifndef _SYS_IEEE488_H
#define _SYS_IEEE488_H
/*
	io control commands and macros
	(B) is used for board io control
	(D) is used for device io control
*/
#define	IEIOC_WBDCFG	0	/* write board configuration (B) */
#define IEIOC_RBDCFG	1	/* read board configuration (B) */
/* #define IEIOC_WMDEVCFG	2	write multiple device configuration (B) */
/* #define IEIOC_RMDEVCFG	3	read multiple device configuration (B) */
#define IEIOC_WSDEVCFG	4	/* write single device configuration (D) */
#define IEIOC_RSDEVCFG	5	/* read single device configuration (D) */
#define IEIOC_WAITSRQ	6	/* wait for SRQ or timeout */
#define IEIOC_UNUSED1	7	/* unused 1 */
#define IEIOC_RBDSTAT	8	/* read board status (B) */
#define	IEIOC_UNUSED2	9	/* unused 2 */
#define IEIOC_BDCMDS	10	/* send a string of commands to board (B)*/
#define IEIOC_DTRIG	11	/* trigger this device (D) */
#define IEIOC_DCLEAR	12	/* selective clear device (D/B) */
#define IEIOC_GETERR	13	/* get last error of this device (D/B) */
#define IEIOC_RDEVSTAT	14	/* get device status (D) */
#define IEIOC_SPOLL	15	/* unconditional spoll (D)*/
#define IEIOC_GETSPRB	16	/* get last spr */
#define	IEIOC_PPOLL	17	/* do a parallel poll (B/D) */
#define	IEIOC_PPE	18	/* parallel poll enable (D) */
#define	IEIOC_PPD	19	/* parallel poll diable */
#define IEIOC_GTL	20	/* goto local function (D) */
#define IEIOC_GETVERS   21      /* get version (D/B) */
#define IEIOC_LINES     22      /* Read gpib lines (B) */

/*
	strukture to configure the ieee488 board
*/
struct ie_board_cnfg{
	unsigned long int	mode;	/* speciall modes; see below */
	unsigned long int	tmo_cmds; /* command timeout [ms] */
	unsigned char	dev_no;		/* number of 3.0 devs */
	unsigned char	log_addr;	/* logical address of board */
	};

/* board modes */
#define IE_BM_DMA	1	/* use dma on all data transfers */
#define IE_BM_FASTCMDS	2	/* ONLY IGPIB: use FIFO on commands */

/*
	NEW:
	generic event mask for select()/poll() or message handling
*/
#define	IE_MAX_EVENTS		8	/* max number of events to handle */

#define IE_EVENT_EOL		0	/* end of event list */
#define	IE_EVENT_SRQ_STB	1	/* (DEV) serial poll status byte */
#define	IE_EVENT_PPOLL		2	/* (DEV) parallel poll status */
#define	IE_EVENT_IOREQ		4	/* (BOARD) io request */
#define	IE_EVENT_STATUS		8	/* (BOARD) status change */

#define	IE_OS_EVENT_READ	0x01	/* may call the read() */
#define IE_OS_EVENT_WRITE	0x02	/* may call the write() */
#define IE_OS_EVENT_EXCEPT	0x04	/* any exceptional condition */
#define IE_OS_EVENT_CLEAR	0x40	/* internally used */
#define IE_OS_EVENT_CLRONIO	0x80	/* clear on OS operation done */

#define IE_STB_BIT2MASK(bmsk,bcmp)	((bmsk) | ((bcmp) << 8))
#define IE_STB_BMSK(x)		((x) & 0xff)
#define IE_STB_BCMP(x)		(((x) >> 8) & 0xff)
#define IE_STB_DOESFIT(x,stb)	(((stb) & IE_STB_BMSK(x)) == IE_STB_BCMP(x))

struct	ie_event_scheme
{
	unsigned char	GPIBevent;	/* one of IE_EVENT_* */
	unsigned char	OSevent;	/* one of IE_OS_EVENT_* */
	unsigned short	mask;		/* depends on event type */
};

/*
	strukture for low level information
*/

#define	MAX_DEVICE	125	/* max number of sys 4.0 devices per board */

struct ie_devdep_cnfg {
	struct ie_event_scheme	evtScm[IE_MAX_EVENTS];
	unsigned long int	tmo_rd;	/* timeout for reading [ms] */
	unsigned long int	tmo_wr;	/* dto for writeing */
	unsigned long int	tmo_sp; /* dto for spoll */
	unsigned long int	mode;
	unsigned char		eos_rd;	/* eosbyte to use on read */
	unsigned char		eos_wr;	/* eosbyte to use on write */
	unsigned char		pad;	/* primary bus address */
	unsigned char		sad;	/* secondary bus address */
	};

/* device modes */

#define IE_DM_WEOI	0x00001	/* set eoi on last byte of write */
#define IE_DM_WEOS	0x00002	/* send EOS on end of string */
#define IE_DM_WEOIEOS	0x00004	/* set eoi on every eos */
#define IE_DM_REOI	0x00008	/* use eoi as terminator */
#define IE_DM_REOS	0x00010	/* use eos as terminator */
#define IE_DM_REOS7	0x00020	/* use 7 Bit of eos only */
/* #define IE_DM_WTMO	0x00040	use COMP instead GLOBAL timeout */
/* #define IE_DM_RTMO	0x00080	dto. for writeing */
#define IE_DM_WATN	0x00100	/* toggle ATN after every write call */
#define IE_DM_RATN	0x00200	/* dto. for reading */
#define IE_DM_HSTIM	0x00400	/* do fast handshake timeing */
#define IE_DM_WLAST	0x00800	/* wait for last byte is written out */
#define	IE_DM_AUNADR	0x01000	/* enable automatic unaddressing */
#define IE_DM_ASPALL	0x02000	/* automatic seriall poll on every SRQ */
#define IE_DM_ASPSUCC	0x04000	/* auto seriall poll til success */
#define IE_DM_SPRDSC	0x08000	/* discard seriall poll respons on io */
#define IE_DM_RSLOW	0x10000	/* ONLY IGPIB: don`t FIFO on READ */
#define IE_DM_WSLOW	0x20000	/* ONLY IGPIB: don`t FIFO on WRITE */

#define	IE_DM_MASK	0x3ffff
#define	IE_DM_ISMODE(m)	(m & IE_DM_MASK)

/*
	status of the interface board
	and the meanings of the bits
*/
struct ie_bd_stat {
	unsigned short int	hwstatus;
	};

#define IE_BST_INIT	0x80	/* interface int initialized */
#define IE_BST_SYSC	0x40	/* interface is system controller */
#define IE_BST_CIC	0x20	/* controller in charge */
#define IE_BST_ATN	0x10	/* ATN is set true */
#define IE_BST_SRQ	0x08	/* the SRQ line is aktive */
#define IE_BST_REM	0x04	/* board is in remote state */
#define IE_BST_LA	0x02	/* board is addressed as listener */
#define IE_BST_TA	0x01	/* board is addressed as talker */
#define IE_BST_LLO	0x100	/* local lockout is aktive */
#define IE_BST_EOI	0x200	/* last read was terminated by EOI */


/*
        GPIB bus lines
        REMARK: The values are the same as the DOS/WINDOWS LN_* values
*/
#define IE_GPLN_DAV     0x01    /* data valid */
#define IE_GPLN_NDAC    0x02    /* not data accepted */
#define IE_GPLN_NRFD    0x04    /* not ready for data */
#define IE_GPLN_IFC     0x08    /* interface clear */
#define IE_GPLN_REN     0x10    /* remote enable */
#define IE_GPLN_SRQ     0x20    /* service requests */
#define IE_GPLN_ATN     0x40    /* attention line */
#define IE_GPLN_EOI     0x80    /* end or identify */

/*
	device status
*/
struct ie_dev_stat {
	unsigned short	status;
	};

#define IE_DST_EOI	0x01	/* EOI was true on last end */
#define IE_DST_SRQ	0x02	/* SRQ is aktive */
#define IE_DST_SPRESP	0x04	/* buffer contains seriall pesponse */

/*
	seriall poll response type
*/
struct ie_spr_byte {
	unsigned char	byte;	/* seriall response byte */
	unsigned char	status;	/* status when byte was read */
	};

#define IE_SPRS_EOI	0x01	/* byte received with EOI true */
#define IE_SPRS_TMO	0x02	/* timeout on spoll, byte illegal */
#define IE_SPRS_NONE	0x04	/* no byte in fifo */


/* don't include this if Version 2.x header was included */
#ifndef IE_ENVR
/* defines to use for IO-Control */

#define		IE_ENVR		0	/* read enviroment */
#define 	IE_ENVW 	1	/* write enviroment */
#define		IE_STATR	2	/* read status */
#define 	IE_ERSTAT	3	/* get information about error */
#define		IE_EOFSET	4	/* use EOI as EOF */

struct ie_errinf {
	short int	operation;	/* operation during error occured */
	short int	reason; 	/* reason for error */
	long int	datapos;	/* datapostition of error */
	};

/*
	definitions for error code (reasons)
*/
#define IEER_NON	0	/* no error */
#define IEER_HWNI	1	/* hardware not initialized */
#define IEER_NSC	2	/* not systemcontroller */
#define IEER_PARL	3	/* illegal parameter length */
#define IEER_TMO	4	/* timeout */
#define IEER_NOLI	5	/* no listener found */
#define IEER_OVFL	6	/* buffer overflow */
#define IEER_MACC	7	/* memory access denied */
#define IEER_MALL	8	/* malloc failed */
#define IEER_ATNU	9	/* atn not comeing up */
#define IEER_ATND	10	/* atn not going down */
#define IEER_DMANA	11	/* DMA not avaiable */
#define IEER_DMAF	12	/* start DMA failed */
#define IEER_ILPR	13	/* illegal paramter */
#define IEER_NOCD	14	/* no interfacecard found */
#define IEER_ILADM	15	/* illegal address mode */
#define IEER_UBRK	16	/* user break */
#define IEER_NCIC       17      /* not controller in charge */

/*
	definitions for operation of error
*/
#define IEEO_DATAR	0	/* read data */
#define IEEO_DATAW	1	/* write data */
#define IEEO_XCMDR	2	/* read extended or commands */
#define IEEO_XCMDW	3	/* write extended */
#define IEEO_OPTR	4	/* read options */
#define IEEO_OPTW	5	/* write options */
#define IEEO_STATR	6	/* read status or error */

#define 	IE_ESCCH	0x1b	/* escape character */
#define		IE_INIT		'I'	/* initialize interface (raw) */
#define		IE_IFC		'i'	/* send interface clear */
#define		IE_PPOLL	'P'	/* force parallel poll */
#define		IE_SRQEN	'S'	/* enable srq */
#define		IE_SRQDI	's'	/* disable srq */
#define		IE_REMEN	'R'	/* set remote enable */
#define		IE_REMDI	'r'	/* set remote disable */
#define		IE_ATNEN	'A'	/* set atn true */
#define		IE_ATNDI	'a'	/* set atn false */
#define 	IE_WSRQ 	'W'	/* wait until srq arrived */

#endif


#ifndef GPIB_CMDS
#define GPIB_CMDS
/* GPIB interface commands */
#define GPIB_GTL	0x01 /* goto local */
#define GPIB_SDC	0x04 /* selective device clear */
#define GPIB_PPC	0x05 /* parallel poll configure */
#define GPIB_GET	0x08 /* group execute trigger */
#define GPIB_TCT	0x09 /* take control */
#define GPIB_LLO	0x11 /* local lockout */
#define GPIB_DCL	0x14 /* generic device clear */
#define GPIB_PPU	0x15 /* parallel poll unconfigure */
#define GPIB_SPE	0x18 /* serial poll enable */
#define GPIB_SPD	0x19 /* serial poll disable */
#define GPIB_LAG	0x20 /* listener address group */
#define GPIB_UNL	0x3f /* unlisten */
#define GPIB_TAG	0x40 /* talker address group */
#define GPIB_UNT	0x5f /* untalk */
#define GPIB_SCA	0x60 /* secondary address group */
#define GPIB_PPE	0x60 /* parallel poll enable */
#define GPIB_PPD	0x70 /* parallel poll disable */
#endif /* GPIB_CMDS */

#endif /* _SYS_IEEE488_H */
